/**
 * Functions to get data from gitlab.cl.uni-heidelberg.de (hosting the protocols repo) using the credentials
 * found in the browser session with gitlab.cl.uni-heidelberg.de.
 *
 * Wait, you might ask, shouldn't the Same Origin Policy
 * (https://developer.mozilla.org/en-US/docs/Web/Security/Same-origin_policy) prevent that?
 * — Yes, mostly.
 *
 * And this "mostly" is also the reason why this script is kinda a gigantic hack.
 *
 * For those who have never heard of the Same Origin Policy before: The general idea is that you
 * do not want a website (say "cute-cat-pics.totallylegit.example") make requests to another website
 * (e.g., "my-online-banking.example" or "private-messaging-service.example") in your name (i.e.
 * using the credentials found in the browser session for that page). Otherwise, the cute cat pics
 * website could steal all your personal messages and authorize money transfers from your account,
 * if you happen to be logged in.
 *
 * This was possible in the early days, found to be a massiv security hole, and patched.
 * However, because websites already started to depend on the mechanism for legitimate use cases, it
 * was decided against completely patching it (which could be achieved by having fully isolated containers
 * for credentials per website; a website would then be allowed to send requests to all other websites,
 * but never using credentials other than its own).
 *
 * This means it is possible to send requests to different websites, re-using their credentials, in certain
 * cases: https://developer.mozilla.org/en-US/docs/Web/Security/Same-origin_policy#cross-origin_network_access
 * Generally, you can send all the requests you want (re-using credentials!), but you cannot read the responses.
 * This would prevent the cute cat pics website from reading all your personal messages, but it could still
 * send money to other bank accounts. This is the reason why CSRF tokens were invented (basically, the bank
 * will only send money to another account when your browser can echo back a random string the bank sent it,
 * which the cute cat pics website could not read).
 *
 * Well, one exeception where you can send requests (re-using credentials) and read the responses is when embdeding
 * images. This means we use an image hosted at gitlab.cl.uni-heidelberg.de and only accessible to logged-in users
 * to determine if the user is logged in.
 */

/**
 * Checks if the browser can access the image at the given URL. This must be a valid image, otherwise
 * the check will always fail.
 *
 * @param {String} imageUrl A url to an image file (likely hosted on some other origin)
 * @returns The <img> object for further processing, or null if the browser cannot access it.
 */
async function access(imageUrl) {
    try {
        return await new Promise((resolve, reject) => {
            const img = document.createElement("img")
            img.addEventListener("load", resolve(img))
            img.addEventListener("error", reject)
            img.src = imageUrl
        })
    } catch {
        return null
    }
}

function toIsoString(year, month, day) {
    return `${year.toString().padStart(4, "0")}-${month.toString().padStart(2, "0")}-${day.toString().padStart(2, "0")}`
}

/**
 * Wait until the image is rendered, i.e. until its width or height are different from 0
 */
async function waitUntilRendered(img) {
    function isRendered(img) {
        return img.width != 0 || img.height != 0
    }

    if (isRendered(img)) return // that was easy

    await new Promise((resolve, _) => {
        const intervalId = setInterval(() => {
            if (isRendered(img)) {
                clearInterval(intervalId)
                resolve()
            }
        }, 10)
    })
}

export function getProtocolEditLink(dateIso) {
    return `https://gitlab.cl.uni-heidelberg.de/-/ide/project/fachschaft/protokolle/edit/master/-/${dateIso.slice(0, 4)}/${dateIso}.md`
}

/**
 * Return an object with metadata about the latest protocol, or null, if the metadata cannot be accessed
 * (e.g., because the user is not logged in to GitLab, is logged in but has no access to the repo, or an error occurred in the pipeline).
 *
 * An example metadata object would be:
 * {
 *   year: 2024,
 *   month: 6,
 *   day: 19,
 *   dateIso: "2024-06-19",
 *   containsResolution: true,
 *   editLink: "https://gitlab.cl.uni-heidelberg.de/-/ide/project/fachschaft/protokolle/edit/master/-/2024/2024-06-19.md",
 *   downloadLink: "https://gitlab.cl.uni-heidelberg.de/api/v4/projects/1193/jobs/artifacts/master/raw/pdfs/2024-06-19.pdf?job=convert-to-pdf",
 *   resolutionDownloadLink: "https://gitlab.cl.uni-heidelberg.de/api/v4/projects/1193/jobs/artifacts/master/raw/pdfs/2024-06-19_nur_Beschlüsse.pdf?job=convert-to-pdf"
 * }
 */
export async function getLatestProtocolMetadata() {
    const img = await access(
        "https://gitlab.cl.uni-heidelberg.de/api/v4/projects/1193/jobs/artifacts/master/raw/prtcl.svg?job=create-protocol-info-svg",
    )

    if (!img) {
        return null
    }

    await waitUntilRendered(img)

    const year = img.width
    const month = Math.floor(img.height / 100)
    const dayAndMaybeOffset = img.height - 100 * month
    const containsResolution = dayAndMaybeOffset >= 43
    const day = containsResolution ? dayAndMaybeOffset - 42 : dayAndMaybeOffset

    const dateIso = toIsoString(year, month, day)

    const editLink = getProtocolEditLink(dateIso)
    const downloadLink = `https://gitlab.cl.uni-heidelberg.de/api/v4/projects/1193/jobs/artifacts/master/raw/pdfs/${dateIso}.pdf?job=convert-to-pdf`
    const resolutionDownloadLink = containsResolution
        ? `https://gitlab.cl.uni-heidelberg.de/api/v4/projects/1193/jobs/artifacts/master/raw/pdfs/${dateIso}_nur_Beschlüsse.pdf?job=convert-to-pdf`
        : null

    return {
        dateIso,
        year,
        month,
        day,
        containsResolution,
        editLink,
        downloadLink,
        resolutionDownloadLink,
    }
}
